""""
SYSTEM REQUIREMENTS:
The following Centos packages are required for M2Crypto:
python3-devel openssl-devel swig gcc-c++
"""
from setuptools import setup

# This is required to allow editable pip installs while using the declarative configuration (setup.cfg)
setup()
